#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QMessageBox>
#include <QFileDialog>
#include <QColorDialog>
#include <QMessageBox>

#include "acercade.h"
#include "brillocontraste.h"
#include "dialognueva.h"
#include "imagenes.h"
#include "mediadevideo.h"
#include "mediaponderada.h"
#include "rotaravideo.h"
#include "suavizados.h"

QString FiltroImagen = "Todos los formatos (*.bmp *.dib *.jpg *.jpeg *.jpe *.tif *.tiff *.png *.gif);;Archivos BMP (*.bmp *.dib);;Archivos JPG (*.jpg *.jpeg *.jpe);;Archivos TIF (*.tif *.tiff);;Archivos GIF (*.gif);;Otros (*.*)";

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    inic_fotos();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::setStatusBarText(QString cadena)
{
    ui->statusBar->showMessage(cadena, 0);
}

int MainWindow::comprobar_primera_libre (void)
{
    int pl= primera_libre();
    if (pl==-1)
        QMessageBox::warning(this, "Error al crear imagen",
                             "Lo siento. No se pueden crear ms ventanas.");
    return pl;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_actionSalir_triggered()
{
    close();
}

void MainWindow::on_actionNueva_imagen_triggered()
{
    int pl= comprobar_primera_libre();
    if (pl!=-1) {
        DialogNueva nueva;
        if (nueva.exec()) {
            CvScalar color= CV_RGB(nueva.red(), nueva.green(), nueva.blue());
            crear_nueva(pl, nueva.getWidth(), nueva.getHeight(), color);
        }
    }
}

void MainWindow::on_actionAbrir_imagen_triggered()
{
    int pl= comprobar_primera_libre();
    if (pl!=-1) {
        QString nombre= QFileDialog::getOpenFileName(this, "Abrir imagen", ".", FiltroImagen);
        if (!nombre.isEmpty())
            crear_nueva(pl, nombre.toAscii());
    }
}

void MainWindow::on_actionPunto_triggered()
{
    herr_actual= puntos;
    ui->toolButton_4->setChecked(true);
}

void MainWindow::on_actionLinea_triggered()
{
    herr_actual= lineas;
    ui->toolButton_5->setChecked(true);
}

void MainWindow::on_actionSeleccionar_triggered()
{
    herr_actual= seleccionar;
    ui->toolButton_6->setChecked(true);
}

void MainWindow::on_toolButton_2_clicked()
{
    on_actionNueva_imagen_triggered();
}

void MainWindow::on_pushButton_clicked()
{
    QColor color= QColorDialog::getColor();
    if (color.isValid()) {
        QString estilo= "background-color: rgb(";
        estilo+= QString::number(color.red())+",";
        estilo+= QString::number(color.green())+",";
        estilo+= QString::number(color.blue())+")";
        ui->pushButton->setStyleSheet(estilo);
        color_pincel= CV_RGB(color.red(), color.green(), color.blue());
    }
}

void MainWindow::on_horizontalSlider_valueChanged(int value)
{
    radio_pincel= value;
}

void MainWindow::on_horizontalSlider_2_valueChanged(int value)
{
    difum_pincel= value;
}

void MainWindow::on_toolButton_clicked()
{
    on_actionAbrir_imagen_triggered();
}

void MainWindow::on_actionGuardar_triggered()
{
    int fa= foto_activa();
    if (fa!=-1)
        guardar_foto(fa);
}

void MainWindow::on_actionGuardar_como_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        QString nombre= QFileDialog::getSaveFileName(this, "Guardar imagen", foto[fa].nombre, FiltroImagen);
        if (!nombre.isEmpty())
            guardar_foto(fa, nombre.toAscii());
    }
}

void MainWindow::on_actionCerrar_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        if (foto[fa].modificada && preguntar_guardar) {
            QString nombre= foto[fa].nombre;
            nombre= "El archivo " + nombre + " ha sido modificado.\nDesea guardarlo?";
            int resp= QMessageBox::question(this, "Archivo modificado", nombre,
                                            QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
            if (resp==QMessageBox::Yes)
                guardar_foto(fa);
            else if (resp==QMessageBox::Cancel)
                return;
        }
        cerrar_foto(fa);
    }
}

void MainWindow::on_actionPreguntar_si_guardar_triggered()
{
    preguntar_guardar= ui->actionPreguntar_si_guardar->isChecked();
}

void MainWindow::on_actionColor_del_pincel_triggered()
{
    on_pushButton_clicked();
}

void MainWindow::on_actionSeleccionar_todo_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        foto[fa].roi= cvRect(-1, -1, -1, -1);
        mostrar(fa);
    }
}

void MainWindow::on_toolButton_4_clicked()
{
    herr_actual= puntos;
}

void MainWindow::on_toolButton_5_clicked()
{
    herr_actual= lineas;
}

void MainWindow::on_toolButton_6_clicked()
{
    herr_actual= seleccionar;
}

void MainWindow::on_actionInvertir_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        int pl= comprobar_primera_libre();
        if (pl!=-1)
            invertir(fa, pl);
    }
}

void MainWindow::on_actionRotar_90_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        int pl= comprobar_primera_libre();
        if (pl!=-1)
            rotar_exacto(fa, pl, 1);
    }
}

void MainWindow::on_actionRotar_180_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        int pl= comprobar_primera_libre();
        if (pl!=-1)
            rotar_exacto(fa, pl, 1);
    }
}

void MainWindow::on_actionRotar_91_triggered()
{
    int fa= foto_activa();
    if (fa!=-1) {
        int pl= comprobar_primera_libre();
        if (pl!=-1)
            rotar_exacto(fa, pl, 1);
    }
}

void MainWindow::on_actionBrillo_contraste_triggered()
{
    if (foto_activa()!=-1) {
        brillocontraste bc;
        bc.exec();
    }
}

void MainWindow::on_actionGausiano_triggered()
{
    if (foto_activa()!=-1) {
        suavizados sg(CV_GAUSSIAN);
        sg.exec();
    }
}

void MainWindow::on_actionMedia_triggered()
{
    if (foto_activa()!=-1) {
        suavizados sg(CV_BLUR);
        sg.exec();
    }
}

void MainWindow::on_actionMedia_ponderada_triggered()
{
    if (foto_activa()!=-1) {
        int pl= comprobar_primera_libre();
        if (pl!=-1) {
            mediaponderada mp;
            mp.exec();
        }
    }
}

void MainWindow::on_actionRotar_imagen_triggered()
{
    if (foto_activa()!=-1) {
        rotaravideo rv;
        rv.exec();
    }
}

void MainWindow::on_actionAcerca_de_triggered()
{
    acercade ad;
    ad.exec();
}

void MainWindow::on_actionImagen_media_triggered()
{
    mediadevideo mv;
    mv.exec();
}
